#include <QtDebug>
#include <QPainter>
#include <QTime>
#include <QGraphicsScene>

#include <opencv/cv.h>
#include <opencv/highgui.h>
#include <opencv/cxcore.h>
#include <sys/time.h>

#include "mainwindow.h"
#include "ui_mainwindow.h"

using namespace cv;

const char* cascade_name ="../cv/haarcascades/haarcascade_frontalface_alt2.xml";
CvPoint pt1, pt2;

IplImage* frame;

// Draw the FPS figure on the image (requires at least 2 calls)
void MainWindow::drawFPS(IplImage *img)
{
  static int counter = 0;
  static clock_t t;
  static float fps;
  char fps_text[20];
  CvFont font;
  cvInitFont(&font,CV_FONT_HERSHEY_SIMPLEX|CV_FONT_ITALIC, 0.5,1.0,1.0,2);

  // Add fps figure (every 10 frames)
  if (counter > 10)
  {
    fps = (10.0f/(clock()-t) * CLOCKS_PER_SEC);
    t=clock();
    counter = 0;
  }

  // Increment counter
  ++counter;

  // Get the figure as a string
  sprintf(fps_text,"FPS: %.2f",fps);

  // Draw the string on the image
  cvPutText (img,fps_text,cvPoint(10,25), &font, cvScalar(0,0,255));
}

void MainWindow::detect_and_draw( IplImage* img )
{

    // Create memory for calculations
    static CvMemStorage* storage = 0;

    // Create a new Haar classifier
    static CvHaarClassifierCascade* cascade = 0;

    int scale = 1;

    // Create a new image based on the input image
    //IplImage* temp = cvCreateImage( cvSize(img->width/scale,img->height/scale), 8, 3 );

    // Create two points to represent the face locations
    int i;

    // Load the HaarClassifierCascade
    cascade = (CvHaarClassifierCascade*)cvLoad( cascade_name, 0, 0, 0 );


    // Allocate the memory storage
    storage = cvCreateMemStorage(0);

    // Create a new named window with title: result
    //cvNamedWindow( "result", 1 );

    // Clear the memory storage which was used before
    cvClearMemStorage( storage );

    // Find whether the cascade is loaded, to find the faces. If yes, then:
    if( cascade )
    {

        // There can be more than one face in an image. So create a growable sequence of faces.
        // Detect the objects and store them in the sequence
        CvSeq* faces = cvHaarDetectObjects( img, cascade, storage,
                                            1.1, 2, CV_HAAR_DO_CANNY_PRUNING,
                                            cvSize(40, 40) );

        // Loop the number of faces found.
        for( i = 0; i < (faces ? faces->total : 0); i++ )
        {
           // Create a new rectangle for drawing the face
            CvRect* r = (CvRect*)cvGetSeqElem( faces, i );

            // Find the dimensions of the face,and scale it if necessary
            pt1.x = r->x*scale;
            pt2.x = (r->x+r->width)*scale;
            pt1.y = r->y*scale;
            pt2.y = (r->y+r->height)*scale;

            // Draw the rectangle in the input image
            cvRectangle( img, pt1, pt2, CV_RGB(255,0,0), 3, 8, 0 );
         }
    }

    // Show the image in the window named "result"
    //cvShowImage( "result", img );

    // Release the temp image created.
    //cvReleaseImage( &temp );
}

int stops=0;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent)
{
    setupUi(this);
    show();
    //setAttribute(Qt::WA_OpaquePaintEvent, true); // don't clear the area before the paintEvent
    //setAttribute(Qt::WA_PaintOnScreen, true); // disable double buffering

    CvCapture* capture = cvCreateCameraCapture(0);

    QImage m_i;

    while(stops!=1)
    {
        IplImage * frame1 = cvQueryFrame(capture);
        detect_and_draw(frame1);
        drawFPS(frame1);
        putImage(frame1);
        cvWaitKey(33);
    }

    //or
    /*
    while(stops!=1)
    {
        IplImage * frame1 = cvQueryFrame(capture);
        //IplImage * frame = cvLoadImage( "/media/data/Additional/Programing/image_compare/img/after/images0001.png");
        cvCvtColor(frame1,frame1,CV_BGR2RGB);
        m_i = QImage((unsigned char *)frame1->imageDataOrigin,frame1->width,frame1->height,QImage::Format_RGB888);
        label->setPixmap(QPixmap::fromImage(m_i,0));
        cvWaitKey(33);
    }
    */

    cvReleaseCapture(&capture);
}

void MainWindow::on_stopButton_clicked()
{
    stops=1;
}

void MainWindow::putImage(IplImage *cvimage)
{    

   QImage image(100,100,QImage::Format_RGB32);
    int cvIndex, cvLineStart;
    // switch between bit depths
    switch (cvimage->depth) {
        case IPL_DEPTH_8U:
            switch (cvimage->nChannels) {
                case 3:
                    if ( (cvimage->width != image.width()) || (cvimage->height != image.height()) ) {
                        QImage temp(cvimage->width, cvimage->height, QImage::Format_RGB32);
                        image = temp;
                    }

                    cvIndex = 0; cvLineStart = 0;
                    for (int y = 0; y < cvimage->height; y++) {
                        unsigned char red,green,blue;
                        cvIndex = cvLineStart;
                        for (int x = 0; x < cvimage->width; x++) {
                            // DO it
                            red = cvimage->imageData[cvIndex+2];
                            green = cvimage->imageData[cvIndex+1];
                            blue = cvimage->imageData[cvIndex+0];

                            image.setPixel(x,y,qRgb(red, green, blue));
                            cvIndex += 3;
                        }
                        cvLineStart += cvimage->widthStep;
                    }
                    break;
                default:
                    printf("This number of channels is not supported\n");
                    break;
            }
            break;
        default:
            printf("This type of IplImage is not implemented in QOpenCVWidget\n");
            break;
    }
    label->setPixmap(QPixmap::fromImage(image));
}

void MainWindow::closeEvent(QCloseEvent * event)
{
    stops=1;
    sleep(1);
    cvReleaseCapture(&capture);
    //quit();
    qApp->quit();
}
